@@ -1,6 +1,6 @@
-;;; eselect-mode.el --- edit eselect files
+;;; eselect-mode.el --- edit eselect files  -*-lexical-binding:t-*-
 
-;; Copyright 2006-2019 Gentoo Authors
+;; Copyright 2006-2023 Gentoo Authors
 
 ;; Author: Matthew Kennedy <mkennedy@gentoo.org>
 ;;	Diego Pettenò <flameeyes@gentoo.org>
@@ -31,6 +31,26 @@
 (require 'sh-script)
 (require 'font-lock)
 
+;;; Variables.
+
+(defgroup eselect nil
+  "Major mode for Gentoo eselect modules."
+  :group 'languages)
+
+(defcustom eselect-mode-fix-whitespace t
+  "If non-nil, delete trailing whitespace before writing a file."
+  :type 'boolean
+  :group 'eselect)
+
+(defcustom eselect-mode-update-copyright t
+  "If non-nil, update copyright years before writing a file."
+  :type 'boolean
+  :group 'eselect)
+
+(defvar eselect-mode-copyright-regexp
+  "^#[ \t]*Copyright[ \t]+\\([1-9][0-9]+\\)\\(?:-\\([1-9][0-9]+\\)\\)?\
+[ \t]+\\(.*\\<Gentoo Authors\\>.*\\)")
+
 ;;; Font-lock.
 
 (defvar eselect-mode-keywords-warn
@@ -69,20 +89,11 @@
      "get_repo_news_dir" "env_update")
     font-lock-type-face))
 
-(defun eselect-mode-make-keywords-list (keywords-list face
-						      &optional prefix suffix)
-  ;; based on `generic-make-keywords-list' from generic.el
-  ;; Note: XEmacs doesn't have generic.el
-  (unless (listp keywords-list)
-    (error "Keywords argument must be a list of strings"))
-  (cons (concat prefix "\\<"
-		(regexp-opt keywords-list t)
-		"\\>" suffix)
-	face))
-
 (defvar eselect-mode-font-lock-keywords
   (mapcar
-   (lambda (x) (apply 'eselect-mode-make-keywords-list x))
+   (lambda (x)
+     (cons (regexp-opt (car x) 'words)
+	   (cadr x)))
    (list
     eselect-mode-keywords-warn
     eselect-mode-keywords-core
@@ -95,21 +106,44 @@
 
 ;;; Mode definitions.
 
-(defun eselect-mode-before-save ()
-  ;;(delete-trailing-whitespace)	; doesn't exist in XEmacs
+(defun eselect-mode-update-copyright ()
+  "Update the copyright notice in the file's header."
   (save-excursion
     (goto-char (point-min))
-    (while (re-search-forward "[ \t]+$" nil t)
-      (delete-region (match-beginning 0) (point))))
+    (let ((case-fold-search nil))
+      (when (re-search-forward eselect-mode-copyright-regexp 400 t)
+	(let* ((y1 (string-to-number (match-string 1)))
+	       (y2 (and (match-string 2)
+			(string-to-number (match-string 2))))
+	       (year (save-match-data (format-time-string "%Y" nil t)))
+	       (y (string-to-number year)))
+	  (if y2
+	      ;; Update range of years
+	      (if (or (> 2005 y1) (>= y1 y2) (> y2 y))
+		  (lwarn 'eselect :warning
+			 "Suspicious range of copyright years: %d-%d" y1 y2)
+		(if (/= y2 y)
+		    (replace-match year t t nil 2)))
+	    ;; Update single year and convert to range if necessary
+	    (if (or (> 2005 y1) (> y1 y))
+		(lwarn 'eselect :warning "Suspicious copyright year: %d" y1)
+	      (if (/= y1 y)
+		  (replace-match (concat "\\1-" year) t nil nil 1)))))))))
+
+(defun eselect-mode-before-save ()
+  (when eselect-mode-fix-whitespace
+    (delete-trailing-whitespace))
+  (when eselect-mode-update-copyright
+    (eselect-mode-update-copyright)
+    ;; call it only once per buffer
+    (set (make-local-variable 'eselect-mode-update-copyright) nil))
   ;; return nil, otherwise the file is presumed to be written
   nil)
 
 ;;;###autoload
-(define-derived-mode eselect-mode shell-script-mode "Eselect"
+(define-derived-mode eselect-mode sh-mode "Eselect"
   "Major mode for .eselect files."
-  (if (featurep 'xemacs)
-      (make-local-hook 'write-contents-hooks))
-  (add-hook 'write-contents-hooks 'eselect-mode-before-save t t)
+  (add-hook 'write-contents-functions 'eselect-mode-before-save t t)
   (sh-set-shell "bash")
   (setq tab-width 4)
   (setq indent-tabs-mode t))
